
NGINX ({{version}}):

user                             {{user}}
worker_processes                 {{worker_processes}}
worker_connections               {{worker_connections}}
keepalive_timeout                {{keepalive_timeout}}
fastcgi_read_timeout             {{fastcgi_read_timeout}}
client_max_body_size             {{client_max_body_size}}
allow                            {{allow}}
