	##
	# Gzip Settings
	##

	gzip on;
	gzip_disable "msie6";

	gzip_vary on;
    gzip_static on;
	gzip_proxied any;
	gzip_comp_level 6;
	gzip_buffers 16 8k;
	gzip_http_version 1.1;
	gzip_types
	application/atom+xml
	application/geo+json
	application/javascript
	application/json
	application/ld+json
	application/manifest+json
	application/rdf+xml
	application/rss+xml
	application/vnd.ms-fontobject
	application/wasm
	application/x-font-opentype
	application/x-font-truetype
	application/x-font-ttf
	application/x-javascript
	application/x-web-app-manifest+json
	application/xhtml+xml
	application/xml
	application/xml+rss
	font/eot
	font/opentype
	font/otf
	image/bmp
	image/svg+xml
	image/vnd.microsoft.icon
	image/x-icon
	image/x-win-bitmap
	text/cache-manifest
	text/calendar
	text/css
	text/javascript
	text/markdown
	text/plain
	text/vcard
	text/vnd.rim.location.xloc
	text/vtt
	text/x-component
	text/x-cross-domain-policy
	text/xml;