#!/usr/bin/env bash
# WordOps Kernel tweaks script
# script path after installation /opt/wo-kernel.sh

# Netdata Memory tweak
echo 1 >/sys/kernel/mm/ksm/run
echo 1000 >/sys/kernel/mm/ksm/sleep_millisecs
# Redis disable transparent_hugepage
echo never > /sys/kernel/mm/transparent_hugepage/enabled
