
MySQL ({{version}}) on {{host}}:

port                             {{port}}
wait_timeout                     {{wait_timeout}}
interactive_timeout              {{interactive_timeout}}
max_used_connections             {{max_used_connections}}
datadir                          {{datadir}}
socket                           {{socket}}
my.cnf [PATH]                    /etc/mysql/conf.d/my.cnf
