Information about {{domain}} ({{domain_type}}):


Nginx configuration	 {{type}} {{enable}}
{{#php_version}}PHP Version		 {{php_version}}{{/php_version}}

{{#ssl}}SSL			 {{ssl}}{{/ssl}}{{#sslprovider}}
SSL PROVIDER		 {{sslprovider}}{{/sslprovider}}{{#sslexpiry}}
SSL EXPIRY DATE		 {{sslexpiry}}{{/sslexpiry}}

access_log		 {{accesslog}}
error_log		 {{errorlog}}
{{#webroot}}Webroot			 {{webroot}}{{/webroot}}

{{#dbname}}
DB_NAME			 {{dbname}}{{/dbname}}
{{#dbname}}DB_USER			 {{dbuser}}{{/dbname}}
{{#dbname}}DB_PASS			 {{dbpass}}{{/dbname}}
{{#tablepref}}table_prefix		 {{tableprefix}}{{/tablepref}}
