# WPSUBDIRECTORY NGINX CONFIGURATION
# DO NOT MODIFY, ALL CHANGES WILL BE LOST AFTER AN WordOps (wo) UPDATE
if (!-e $request_filename) {
  # Redirect wp-admin to wp-admin/
  rewrite /wp-admin$ $scheme://$host$uri/ permanent;
  # Redirect wp-* files/folders
  rewrite ^(/[^/]+)?(/wp-.*) $2 last;
  # Redirect other php files
  rewrite ^(/[^/]+)?(/.*\.php) $2 last;
}
